import stan        #  pystan --> stan
import matplotlib.pyplot as plt
import scipy.stats as scst
import numpy as np

RawData = [ ['City',     'Temperature', 'Latitude'], 
            ['Sapporo',     15,             43.1],
            ['Sendai',      20,             38.3],
            ['Niigata',     17,             37.9],
            ['Kanazawa',    18,             36.6],
            ['Tokyo',       24,             35.7], 
            ['Osaka',       22,             34.7],
            ['Fukuoka',     22,             33.6],
            ['Kochi',       24,             33.6],
            ['Kagoshima',   23,             31.6],
            ['Naha',        26,             26.2]
]

ID = []
Y = []
X = []
N = len(RawData) - 1
for i in range(N):
    ID.append(RawData[i + 1][0])
    Y.append(RawData[i + 1][1])
    X.append(RawData[i + 1][2])

for v in zip(ID, Y, X):
    print(v)

Data = {'N': N, 'Y': Y, 'X': X}

#sm = pystan.StanModel(file = 'Regression.stan')
#fit = sm.sampling(data = Data, n_jobs = 1)

with open('Regression.stan', 'r') as f_stan:
    sm = stan.build(f_stan.read(), data = Data)
    fit = sm.sample()
    
print(fit)
f = open('fit.txt', 'w')
f.write('fit = \n{}\n'.format(fit))
f.close()

fit = fit.to_frame()

Sampleb0 = fit['b0']
print('len(Sampleb0) = ', len(Sampleb0))
plt.title('Samples of b0')
plt.xlabel('b0')
plt.ylabel('frequency')
plt.hist(Sampleb0)
plt.show()

Sampleb1 = fit['b1']
nb1 = len(Sampleb1)
print('len(Sampleb1) = ', nb1)
npos = 0
for v in Sampleb1:
    if v > 0.0:
        npos += 1
plt.title(('Samples of b1\n') +
          ('Prop(b1 > 0) = {0:.3f}'.format(npos/nb1)))
plt.xlabel('b1')
plt.ylabel('frequency')
plt.hist(Sampleb1)
plt.show()


Samplesgm = fit['sgm']
plt.title('Samples of sgm\n')
plt.xlabel('sgm')
plt.ylabel('frequency')
plt.hist(Samplesgm)
plt.show()

r, p = scst.pearsonr(Sampleb0, Sampleb1)
print('r = ', r)
plt.xlabel('b0')
plt.ylabel('b1')
plt.title('r = {0:.3f}'.format(r))
plt.plot(Sampleb0, Sampleb1, 'b.')
plt.show()


#PredYs = fit['mu']
YsArray = []

for i in range(N):
    YsArray.append(fit[f'mu.{i+1}'])
    
#for i in range(10):
#    YsArray.append([])
#for v in PredYs:
#    for i in range(10):
#        YsArray[i].append(v[i])
MedPreds = []
for i in range(10):
    MedPreds.append(np.percentile(YsArray[i], 50))

plt.xlabel('Data')
plt.ylabel('Prediction')
plt.plot(Y, MedPreds, 'bo')
for nm, y, pred in zip(ID, Y, MedPreds):
    plt.text(y + 0.1, pred + 0.1, nm)
plt.plot([14, 28], [14, 28], 'b-')

plt.show()
